@@ -1,6 +1,7 @@
 import os
 from datetime import datetime
 from time import sleep
+from typing import Union
 
 import pandas as pd
 import numpy as np
@@ -27,8 +28,8 @@ def download_data(ticket: str, ts_start: int, ts_end: int = 0,
     for row in data_generator:
         rows.append(row)
     df = pd.DataFrame(rows, columns=titles)
-    df = df[['close_time', 'close', 'high', 'low', 'volume']]
-    df[['close', 'high', 'low', 'volume']] = df[['close', 'high', 'low', 'volume']].astype(np.float64)
+    df = df[['close_time', 'close', 'open', 'high', 'low', 'volume']]
+    df[['close', 'open', 'high', 'low', 'volume']] = df[['close', 'open', 'high', 'low', 'volume']].astype(np.float64)
     return df
 
 
@@ -93,9 +94,9 @@ def cal_ma_by_other_timeframe(df: pd.DataFrame, ticket: str, window: int,
     return pd.Series(ma_arr)
 
 
-def download_futures_trades(ticket: str, start_date: str = '2020-10-01 00:00:00'):
+def download_futures_trades(ticket: str, start_date: str = '2020-10-01 00:00:00', end_date=None):
     """ Скачивание сделок по фьючерсам """
-    downloader = _FuturesTradesDownloader(ticket, start_date)
+    downloader = _FuturesTradesDownloader(ticket, start_date, end_date)
     downloader.download_futures_trades()
 
 
@@ -106,11 +107,14 @@ class _FuturesTradesDownloader:
     COLUMNS = ['a', 'p', 'q', 'T', 'm']
     MAX_REQ_TO_MIN = 500
 
-    def __init__(self, ticket: str, start_date: str):
+    def __init__(self, ticket: str, start_date: str, end_date: Union[str, None] = None):
         self.client = Client('', '')
         self.ticket = ticket
         self.filename = f'data/{ticket}_trades.csv'
         self.start_ts_ms = int(datetime.strptime(start_date, '%Y-%m-%d %H:%M:%S').timestamp() * 1000)
+        self.end_date_ms = None
+        if end_date is not None:
+            self.end_date_ms = int(datetime.strptime(end_date, '%Y-%m-%d %H:%M:%S').timestamp() * 1000)
 
     def download_futures_trades(self):
 
@@ -137,6 +141,8 @@ class _FuturesTradesDownloader:
             if len(data) == 0:
                 break
             last_id = self._save_data(data)
+            if self.end_date_ms is not None and data[-1]['T'] > self.end_date_ms:
+                break
             if req_to_min % 3 == 0:
                 sleep(0.5)
 
